#include <alsa/asoundlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "wav.h"

FILE *fp = NULL;
struct stat file_stat;

char *audio = NULL;
char *pos = NULL;

int wav_load_file(const char* filename){
	if ( (fp = fopen(filename, "r")) != NULL)
	{
		if (stat(filename, &file_stat) != 0){
			fprintf(stderr, "Cannot determine file size!\n"); 
			return -1; // error while getting file attributes
		}

		audio = malloc(file_stat.st_size);
				
		fread(audio, file_stat.st_size, 1, fp);
		if (ferror(fp) != 0) {
			fclose(fp);
			fprintf(stderr, "Cannot read from file %s!\n", filename); 
			return -2; // failed to read from file
		}
		fclose(fp);
		pos = audio + SIZE_WAVHDR;
		return 0; // success
	}
	fprintf(stderr, "Cannot open file %s!\n", filename); 
	return -3; // failed to open file
}

int wav_close_file(){
	free(audio); audio = NULL;
	return 0;
}

int wav_get_resolution(){
	return (int) *( (uint16_t *) (audio + 34));
}

int wav_get_sampling_rate(){
	return (int) *( (uint32_t *) (audio + 24));
}

int wav_get_channels(){
	return (int) *( ( uint16_t *) (audio + 22));
}

snd_pcm_format_t wav_get_sound_format(){
	uint16_t resolution;	
	resolution = *( (uint16_t *) (audio + 34));
	 
	if (resolution == 8){
	  	/** Signed 8 bit */
		return SND_PCM_FORMAT_U8;
	} else if (resolution == 16) {
		if (*(audio + 3)  == 'F' ) {
			/** Signed 16 bit Little Endian */
			return SND_PCM_FORMAT_S16_LE;
		} else if (*(audio + 3)  == 'X'){
			/** Signed 16 bit Big Endian */
			return SND_PCM_FORMAT_S16_BE;
		}
	}
	// else 
		printf("OOPS, UNKNOWN FILE!\n");
		return SND_PCM_FORMAT_UNKNOWN;
}

int wav_fill_buffer(char* buf, unsigned int bytes){
	
	if (pos >= audio + file_stat.st_size){
		fprintf(stderr, "Nothing left to play!\n");
		return 0;
	}

	if (pos + bytes > audio + file_stat.st_size) {
		bytes = audio + file_stat.st_size - pos;
	}
	memcpy(buf, pos, bytes);
	pos = pos + bytes;
	return bytes;
}
